package Web;

use strict;
use IO::Socket::INET;

our $VERSION = 0.1;

sub new
{
	my $self = $_[0];
	my $obj = bless {}, $self;
	return $obj;
}

sub set_cookie
{
	my $self = shift;
	my($cookie) = @_;

	$self->{cookie} = $cookie;
}

sub proxy
{
	my $self = shift;
	my($proxy, $proxy_host, $proxy_port) = @_;

	$self->{proxy} = $proxy ? $proxy : 0;
	$self->{proxy_host} = $proxy_host ? $proxy_host : '';
	$self->{proxy_port} = $proxy_port ? $proxy_port : '';
}

sub request
{
	my $self = shift;
	my ($method, $host, $port, $uri, $payload, $limit) = @_;
	
	my $header =
	"$method $uri HTTP/1.0\r\n".
	"Host: $host\r\n".
	"User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1b4pre) Gecko/20090401 Firefox/3.5b\r\n".
	"Connection: close\r\n";
	
	$header .= "Cookie: ".$self->{cookie}."\r\n" if $self->{cookie};
	
	if($method eq 'GET')
	{
		$header .= "\r\n";
	}
	elsif(defined $payload && $method eq 'POST')
	{
		$header .=
		"Content-Type: application/x-www-form-urlencoded\r\n".
		"Content-Length: ".length($payload)."\r\n\r\n".$payload;
	}
	
	$self->{host} = $host;
	$self->{port} = $port;
	
	my $socket = new IO::Socket::INET 
	(
		PeerAddr => $self->{proxy} ? $self->{proxy_host} : $self->{host},
		PeerPort => $self->{proxy} ? $self->{proxy_port} : $self->{port},
		PeerProto => 'tcp', 
		TimeOut => 20,
	);

	if($socket)
	{
		return if $self->{proxy} == 2 && !socks($self, $socket);
		syswrite $socket, $header;
		my $answ = 0;
		
		if($limit)
		{
			sysread $socket, $answ, $limit;
		}
		else
		{
			while(sysread $socket, my $buf, 1024)
			{
				$answ .= $buf;
			}
		}

		close $socket;		
		return $answ;
	}
	else
	{
		warn "Can't connect to ".($self->{proxy} ? $self->{proxy_host} : $self->{host})."\n";
	}
}

sub socks
{
	my($self, $socket) = @_;

	my $hex = pack 'H*', '05020002';
    syswrite $socket, $hex;
	sysread $socket, my $result, 4;
	
	return undef if '0500' != unpack 'H*', $result;
    my $len_h = chr length $self->{host};
    $hex = pack('H*','05010003').$len_h.$self->{host}.pack('n',$self->{port});
	
    syswrite $socket, $hex;
	sysread $socket, $result, 100;
	
	$result = substr unpack('H*', $result), 3, 1;
	
	return $result ? undef : 1;
}

1;