use strict;
use warnings;
use threads;
use threads::shared;
use AntiCaptcha;
use Common;
use iStr;
use Web;

my %config = loadcfg('config.cfg');
my %string = loadcfg('string.dat');

$| = 1;
my @trl = ();
my %stat : shared = (ok => 0, error => 0, captcha => 0, unknown => 0);

my @alist : shared = loadf($config{'accounts_file'});
my @tlist = loadf($config{'messages_file'});
my @llist = loadf($config{'links_file'});
my @plist = loadf($config{'proxy_file'});
my @smile = split /,/, $string{'smiles'};

my($tsize, $lsize) = (scalar @tlist, scalar @llist);

$trl[$_] = threads->create(\&main) for 0..$config{'threads'} - 1;
$_->join for @trl;

sub main
{
	my $web = new Web;
	while(@alist)
	{
		if($config{'use_proxy'})
		{
			sleep 1 until proxy($web);
		}
		
		my ($prefix, $cred) = auth($web);
		msleep(0.5);
		if($prefix && $cred)
		{
			if($config{'send_type'} == 1 || $config{'send_type'} == 2)
			{
				my $uri = '/dk?st.cmd=userFriend';
				$uri .= 2 if $config{'send_type'} == 2;
				
				my $page = $web->request('GET', $prefix.'.odnoklassniki.ru', 80, $uri, '', 0);
				if($page =~ /$string{'users_number'}/o)
				{
					my $number = $1;
					cout("  Got $number users [$cred]");
					
					my $page_count = int $number/20 + 0.99;
					
					for(my $i=1; $i<=$page_count; $i++)
					{
						cout("  Page $i [$cred]");
						$page = $web->request('GET', $prefix.'.odnoklassniki.ru', 80, $uri.'&st.page='.$i, '', 0);
						($page) = ($page =~ /$string{'users_precise'}/so);
						pre_message($web, $prefix, $cred, $page);
					}
				}
				else
				{
					cout("  Can't get users number [$cred]");
				}
			}
			elsif($config{'send_type'} == 3)
			{
				my $page = $web->request('GET', $prefix.'.odnoklassniki.ru', 80, '/dk?st.cmd=userFriendLive&st._aid=TopMenu_FriendsLive', '', 0);
				($page) = ($page =~ /$string{'users_o_precise'}/so);
				pre_message($web, $prefix, $cred, $page);
			}
		}
		msleep(0.5);
	}
}

sub pre_message
{
	my($web, $prefix, $cred, $page) = @_;
	
	if($page)
	{
		my @users = ($page =~ /$string{'users_pages'}/go);
		if(@users)
		{
			for(my $j=0; $j<=$#users; $j+=2)
			{
				my $user = substr $users[$j], -18;
				cout("   Sending to $user  [$cred]");
				my ($uid, $name) = ($users[$j], $users[$j+1]);
				my $code = message($web, $prefix, $uid, $name, $cred);
				if($code == 1)
				{
					inc('ok');
					cout("   sent to $user  [$cred]");
				}
				else
				{
					inc('error');
					cout("   not sent to $user  [$cred]");
				}
				sleep $config{'delay'};
			}
		}
		else
		{
			cout("  Can't get users list [$cred]");
			debug($page) if $config{'verbose'};
		}
	}
	else
	{
		cout("  Empty page data [$cred]");
		debug($page) if $config{'verbose'};
	}
}

sub message
{
	my ($web, $prefix, $uid, $name, $cred) = @_;
	
	my $page = $web->request('GET', $prefix.'.odnoklassniki.ru', 80, '/dk?cmd=PopLayer&st.cmd=friendMain&st.friendId='.$uid.'&st.layer.cmd=PopLayerMsgPersonal&st.layer.referenceId='.$uid.'&st._aid=FriendTopCardButtonsSendMessage', '', 0);
	
	if($page =~ /$string{'send_layer'}/o)
	{
		my $text = text($name);
		my $data = "st.layer.posted=$1&st.layer.error=&st.layer.redirect=&st.layer.msgtrans=&st.layer.ref=$uid&st.layer.referenceId=$uid&st.layer.eventId=&st.layer.eventExt=&st.layer.msg=$text&button_save=%D0%9E%D1%82%D0%BF%D1%80%D0%B0%D0%B2%D0%B8%D1%82%D1%8C";
		$page = $web->request('POST', $prefix.'.odnoklassniki.ru', 80, '/dk?cmd=PopLayer&st.cmd=friendMain&st.friendId='.$uid.'&st.layer.cmd=PopLayerMsgPersonal&st.layer.referenceId='.$uid, $data, 0);
		
		return analyze($web, $prefix, $page, $cred);
	}
	elsif($page =~ /$string{'friend_restricted'}/o)
	{
		cout("  Can't send message [friend_restricted] [$cred]");
	}
	else
	{
		debug($page) if $config{'verbose'};
		cout("  Can't get 'layer.posted' [$cred]");
	}
	
	return 0;
}

sub analyze
{
	my ($web, $prefix, $page, $cred) = @_;
	
	{
		if($page =~ /$string{'message_good'}/o)
		{
			return 1;
		}
		elsif($page =~ /$string{'bad_text'}/o)
		{
			cout("  Can't send message [text_filter] [$cred]");
		}
		elsif($page =~ /$string{'captcha_detect'}/o)
		{
			inc('captcha');
			cout("  Captcha detected [$cred]");
			$page = captcha($web, $prefix, $page, $cred);
			redo;
		}
		elsif($page =~ /$string{'message_step'}/o)
		{
			return 1;
		}
		else
		{
			debug($page) if $config{'verbose'};
		}
	}
	
	return 0;
}

sub auth
{
	my ($web) = @_;
	
	unless(@alist)
	{
		cout('No more accounts');
		threads->exit->detach;
	}
	
	$web->set_cookie();
	my $page = $web->request('GET', 'odnoklassniki.ru', 80, '/', '', 0);

	if($page =~ /$string{'auth_page'}/o)
	{
		my ($prefix, $host, $token) = ($1, $2, $3);	
		my($login, $passw) = split /$config{'delimiter'}/, shift @alist;
		if($login && $passw)
		{
			my $data = "st.posted=set&st.email=$login&st.password=$passw&st.fJS=&st.screenSize=&st.flashVer=&button_go=%D0%92%D0%BE%D0%B9%D1%82%D0%B8";
			$page = $web->request('POST', $prefix.'.'.$host, 80, '/dk?cmd=AnonymLogin&st.cmd=anonymLogin&tkn='.$token, $data, 0);
			my $cookies = join ' ', ($page =~ /Set-Cookie: (.+?;|\r)/igs);

			if($cookies && $page =~ /$string{'good_acc'}/o)
			{
				cout(" [+] $login:$passw");
				$web->set_cookie($cookies);
				return $prefix, "$login:$passw";
			}
			elsif($page =~ /$string{'bad_acc'}/o)
			{
				cout(" [-] $login:$passw [bad]");
			}
			elsif($page =~ /$string{'temp_block'}/o)
			{
				cout(" [-] $login:$passw [block]");
			}
			else
			{
				debug($page) if $config{'verbose'};
				cout(" [-] $login:$passw [bad]");
			}
		}
		else
		{
			cout('Malformed account line');
		}
		
	}
	else
	{
		debug($page) if $config{'verbose'};
		cout('Can\'t get login url');
	}
	
	return 0;
}

sub text
{
	my ($name) = @_;
	
	if($name)
	{
		($name, undef) = split /\s/, $name;
	}
	else
	{
		$name = ' ';
	}
	
	my ($gif, $alt) = split /__/, $smile[rand @smile];
	my %shash = 
	(
		link => $llist[rand $lsize], 
		digit => rnd(1,100), 
		trash => genran(4),
		name => $config{'friendly_name'} ? friendly_name($name) : $name,
		smile => '<img src="http://stg.odnoklassniki.ru/res/default/Images/smiles/'.$gif.'.gif" ondragstart="return ="return false" onresizestart="return false" onresizeend="return false" alt="'.$alt.'">',
	);
	
	my $text = $tlist[rand $tsize];
	$text = mix_str($text);
	$text = replace_char($text, 5) if $config{'letter_replace'};
	$text =~ s/<<(\w+)>>/$shash{$1}/g;
	undef %shash;

	return $text;
}

sub proxy
{
	my ($web) = @_;
	
	unless(@plist)
	{
		cout('No more proxies in the list');
		threads->exit->detach;
	}
	
	my $seed = int rand scalar @plist;
	my ($proxy, $port) = split /:/, $plist[$seed];
	if($proxy && $port)
	{
		$web->proxy($config{'proxy_type'}, $proxy, $port);
		my $test = $web->request('GET', 'odnoklassniki.ru', 80, '/', '', 0);
		if($test =~ /$string{'valid_proxy'}/o)
		{
			cout('Using proxy '.$plist[$seed]);
			return 1;
		}
		else
		{
			debug($test) if $config{'verbose'};
			cout('Bad proxy '.$plist[$seed]);
		}
	}
	else
	{
		cout('Malformed proxy line');
	}
	splice @plist, $seed, 1;
	
	return 0;
}

sub cout 
{
	my ($str) = "@_ [thread ".threads->tid()."]";
	$str .= " " x (78 - length $str) if length $str < 79;
	print $str."\n";
	$str = '';
	$str .= '[ '.$_.' - '.$stat{$_}.' ]' for keys %stat;
	print $str.'[threads: '.(scalar threads->list() + 1)."]\r";
}

sub inc
{
	lock %stat;
	$stat{$_[0]} += $_[1] ? $_[1] : 1;
}

sub captcha
{
	my ($web, $prefix, $page, $cred) = @_;
	($page) = ($page =~ /$string{'captcha_redirect'}/o);
	$page = $web->request('GET', $prefix.'.odnoklassniki.ru', 80, $page, '', 0);
	
	if($page =~ /$string{'captcha_form'}/o)
	{
		my $form = $1;
		my ($img) = ($web->request('GET', $prefix.'.odnoklassniki.ru', 80, '/captcha?st.cmd=captcha', '', 0) =~ /\r\n\r\n(.*)/s);
		
		if($img)
		{
			my $answ = 0;
			my $ac = AntiCaptcha->new($config{'acurl'}, $config{'ackey'});
			my $id = $ac->Recognize($img, 1);
			if($id)
			{
				sleep 3 until($answ = $ac->GetStatus($id));
			}
		
			my $data = "fr.posted=set&fr.code=$answ&button_save=%D0%9F%D0%BE%D0%B4%D1%82%D0%B2%D0%B5%D1%80%D0%B4%D0%B8%D1%82%D1%8C";
			$page = $web->request('POST', $prefix.'.odnoklassniki.ru', 80, $form, $data, 0);
		
			return $page;
		}
		else
		{
			cout("  Can't get captcha body [$cred]");
		}
	}
	else
	{
		cout("  Can't get captcha [form info] [$cred]");
	}
	
	return 0;
}

#kaimi.ru
